@@ -24,6 +24,9 @@
 #include <dirent.h>
 #include <string.h>
 #include <stdlib.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
 
 #include "cdlabel.h"
 #include "common_gtk.h"
@@ -35,10 +38,12 @@
 #include "clist_common.h"
 #include "mainwindow.h"
 #include "contractions.h"
+#include "command.h"
 #include "globals.h"
 
 char cdlabel_tmp_out[L_tmpnam], cdlabel_tmp_items[L_tmpnam];
 char *cdlabelgen_path = NULL;
+char *lpr_path = NULL;
 
 static GtkWidget *item_text_col, *print_file_entry, *eps_filename, *use_eps;
 static GtkWidget *no_date_radio, *date_radio, *cat_entry, *subcat_entry;
@@ -182,30 +187,36 @@ static void path_for_tray_file(GtkWidget *widget, gpointer data) {
 
 static int cdlabeltext_to_file(void) {
 	char *items;
-	FILE *file;
+	int fd;
 
-	if ((char)0 == cdlabel_tmp_items[0])
-		if (NULL == tmpnam(cdlabel_tmp_items)){
-			g_warning("cdlabel.c::cdlabeltext_to_file: could not create "
-					"tempfilename");
+	if ((char)0 == cdlabel_tmp_items[0]) {
+		strncpy(cdlabel_tmp_items, "/tmp/gcombustXXXXXX", L_tmpnam);
+		if (-1 == (fd = mkstemp(cdlabel_tmp_items))) {
+			g_warning("%s::%i: could not create tempfilen for cdlabelgen",
+					__FILE__, __LINE__);
 			return 1;
 		}
-	if (NULL == (file = fopen(cdlabel_tmp_items, "w"))) {
-			g_warning("cdlabel.c::cdlabeltext_to_file: could not fopen %s",
-					cdlabel_tmp_items);
+	} else {
+		if (-1 == (fd = open(cdlabel_tmp_items, O_WRONLY))) {
+			g_warning("%s::%i: could not open tempfilen for cdlabelgen",
+					__FILE__, __LINE__);
 			return 1;
+		}
 	}
-
 	items = gtk_editable_get_chars(GTK_EDITABLE(item_text_col), 0,
 			gtk_text_get_length(GTK_TEXT(item_text_col)));
 	items = strdup(items);
+	if (0 == strlen(items))
+		string_append(items, " ");
 	if ('\n' != items[strlen(items)-1])
 		items = string_append(items, "\n");
-	fprintf(file, items);
+
+	write(fd, items, strlen(items));
+
 	free(items);
 
-	if (EOF == fclose(file))
-		g_warning("cdlabel.c::cdlabel_text_to_file: fclose() returned EOF");
+	if (-1 == close(fd))
+		g_warning("%s::%i: close() returned -1", __FILE__, __LINE__);
 	return 0;
 }
 
@@ -217,6 +228,9 @@ static int execute_cdlabelgen(GtkWidget *widget, gpointer data) {
 	if (cdlabeltext_to_file())
 		return 1;
 
+	if (check_if_exe_ok(cdlabelgen_path))
+		return 1;
+
 	cmd = string_append(cdlabelgen_path, NULL);
 
 	cmd = string_append(cmd, " -c ");
@@ -278,17 +292,19 @@ static int execute_cdlabelgen(GtkWidget *widget, gpointer data) {
 	cmd = string_append(cmd, " -o ");
 	tmp = gtk_entry_get_text(GTK_ENTRY(print_file_entry));
 	if (0 == strlen(tmp)) { /* want gcombust to create a tempfile */
-		if ((char)0 != cdlabel_tmp_out[0])
-			/* if we already have a tempfile, use it */
-			tmp = cdlabel_tmp_out;
-		else if (NULL == (tmp = tmpnam(cdlabel_tmp_out))) {
-			/* else create one */
-			cdlabel_tmp_out[0] = (char)0;
-			g_warning("cdlabel.c::execute_cdlabelgen: "
-					"could not generate tempfile for output");
-			free(cmd);
-			return 1;
+		if ((char)0 == cdlabel_tmp_out[0]) {
+			/* don't have a tempfile, create it */
+			int fd;
+			strncpy(cdlabel_tmp_out, "/tmp/gcombustXXXXXX", L_tmpnam);
+			if (-1 == (fd = mkstemp(cdlabel_tmp_out))) {
+				g_warning("%s::%i: could not create tempfilen for output",
+						__FILE__, __LINE__);
+				free(cmd);
+				return 1;
+			}
+			close(fd);
 		}
+		tmp = cdlabel_tmp_out;
 	}
 	cmd = string_append(cmd, tmp);
 	printf("%s\n", cmd);
@@ -302,7 +318,10 @@ static int execute_cdlabelgen(GtkWidget *widget, gpointer data) {
 
 static void do_print_command(char *program) {
 	char *cmd, *ps_file;
-
+/*
+	if (check_if_exe_ok(program))
+		return;
+*/
 	if (execute_cdlabelgen(NULL, 0))
 		return;
 
@@ -329,7 +348,9 @@ static void execute_gv_preview(GtkWidget *widget, gpointer data) {
 }
 
 static void execute_cdlabel_print(GtkWidget *widget, gpointer data) {
-	do_print_command("lpr");
+	if (check_if_exe_ok(lpr_path))
+		return;
+	do_print_command(lpr_path);
 }
 
 void print_cmd_callback(GtkWidget *widget, gpointer data) {
@@ -463,7 +484,7 @@ void print_cmd_callback(GtkWidget *widget, gpointer data) {
 	date_entry = gtk_entry_new();
 	gtk_widget_show(date_entry);
 	gtk_tooltips_set_tip(tooltips, date_entry,
-			_("If nothing is entered, todays date will be used"), NULL);
+			_("If nothing is entered, today's date will be used"), NULL);
 	gtk_box_pack_start(GTK_BOX(hbox8), date_entry, FALSE, TRUE, 0);
 
 	hbox8 = gtk_hbox_new(FALSE, 0);
@@ -477,14 +498,14 @@ void print_cmd_callback(GtkWidget *widget, gpointer data) {
 				"Suppresses printing of the Plaque (the category/subcategory banner)"
 				"on the traycard, "
 				"thus allowing you to fit even more items on the traycard."
-				"(needs cdlabelgen 1.5 or greater)"), NULL);
+				"(needs cdlabelgen 1.5 or newer)"), NULL);
 
 	word_wrap = gtk_check_button_new_with_label(_("Enable word wrapping"));
 	gtk_widget_show(word_wrap);
 	gtk_box_pack_start(GTK_BOX(hbox8), word_wrap, FALSE, FALSE, 0);
 	gtk_tooltips_set_tip(tooltips, word_wrap, _(
 				"Enables word wrapping of the items that print on the traycard."
-				"(needs cdlabelgen 1.5 or greater)"), NULL);
+				"(needs cdlabelgen 1.5 or newer)"), NULL);
 
 	/* */
 	frame = make_frame(_("EPS-file options"), 3);
@@ -626,7 +647,7 @@ void print_cmd_callback(GtkWidget *widget, gpointer data) {
 	gtk_signal_connect(GTK_OBJECT(preview), "clicked",
 			GTK_SIGNAL_FUNC(execute_gv_preview), NULL);
 	gtk_tooltips_set_tip(tooltips, preview,
-			_("View a preview of the cover with gv"), NULL);
+			_("Preview the cover with gv"), NULL);
 
 	print = gtk_button_new_with_label(_("Print"));
 	gtk_widget_show(print);
