@@ -17,14 +17,11 @@
 */
 
 config_cdr_data *create_cdrecord_options(GtkWidget *);
-void select_speed(GtkWidget *, GtkWidget *speed[]);
+GtkWidget *select_speed(GtkWidget *);
 
 extern GtkWidget *dummy, *eject, *pad, *swab_audio, *dao, *no_image, *use_image, *audio_tracks;
 extern GtkWidget *audio_tracks_combo, *image_path, *copy_iso_cd, *iso_device_path, *copy_audio_cd;
-extern GtkWidget *blank_all, *blank_fast, *audio_device_path;
-extern GtkWidget *fsentry, *need_track_size;
-extern GtkWidget *speed_x[];
-extern char *speed_string[];
+extern GtkWidget *blank_all, *blank_fast, *audio_device_path, *disable_fixate;
+extern GtkWidget *fsentry, *need_track_size, *enable_burnproof, *speed_spin;
 extern char *def_image_path;
-enum n_speed {NO_SPEED = 5};
 
