@@ -18,6 +18,7 @@
 
 gint resize_clist_col_ratio(GtkWidget *, GdkEvent *, gpointer);
 void choose_file(GtkWidget *, gpointer);
+void choose_file_at(char *, gpointer);
 
-GList* gnome_uri_list_extract_filenames(const gchar*);
+GList* gnome_uri_list_extract_filenames(const gchar*, const gint);
 void gnome_uri_list_free_strings(GList *);
