@@ -28,6 +28,7 @@ typedef struct {
 	GtkWidget *lun;
 	GtkWidget *driver;
 	GtkWidget *device;
+	GtkWidget *fifo;
 } config_cdr_data;
 extern config_cdr_data *config_cdr(int);
 
@@ -39,5 +40,9 @@ typedef struct {
 void set_notebook_tab(GtkWidget *, gint, GtkWidget *);
 void choose_path(GtkWidget *, gpointer);
 void alert_user_of_error(gchar *);
+void alert_user_of_error_msg(gchar *, gchar *);
 GtkWidget *make_frame(gchar *, gint);
 void close_dialog(GtkWidget *, gpointer);
+
+void cursor_wait(GtkWidget *);
+void cursor_reset(GtkWidget *);
