@@ -17,7 +17,8 @@
 */
 
 #define SIZE_BUF_SIZE 9
-#define MAX_ROWS 100
+
+enum { MAX_ROWS=1000 };
 
 extern long int totalsize, sectorsize;
 extern GList *exclude_pattern_list;
@@ -28,6 +29,5 @@ extern GtkWidget *mkisofs_follow_symlinks, *mkisofs_include_all, *mkisofs_includ
 
 void mark_size_inaccurate(void);
 gint resize_clist_col_ratio(GtkWidget *, GdkEvent *, gpointer);
-void select_all(GtkWidget *, gpointer);
 void create_filepicker(GtkWidget *, config_cdr_data *);
 
