@@ -17,7 +17,8 @@
 */
 
 typedef struct {
-	char *path;
+	char *realpath;
+	char *userpath;
 	long int size; /* size in sectors */
 } file_data;
 
@@ -25,6 +26,7 @@ void recalc_size(GtkWidget *, gpointer);
 void selected_file(GtkWidget *, gpointer);
 void del_selected_rows(GtkWidget *, gpointer);
 void execute_recalc_size(cmd_v *);
+long int get_old_session_size(config_cdr_data *);
 /*
 void selected_dir(GtkWidget *, gpointer);
 void remove_dir(GtkWidget *, gpointer);
