@@ -24,6 +24,7 @@
 #include "preferences.h"
 #include "vector_commands.h"
 #include "common_gtk.h"
+#include "modify_file_set.h"
 #include "command.h"
 #include "rip_audio.h"
 #include "cdlabel.h"
@@ -32,16 +33,23 @@
 #include "config_files.h"
 #include "detect_scsi.h"
 #include "cdrecord_options.h"
+#include "selection_IO.h"
+#include "multisession.h"
 #include "globals.h"
 
 /* TODO: path and version check */
 
-GtkWidget *prefs_speed[NO_SPEED];
-misc_preferences misc_prefs = { TRUE, TRUE, TRUE, FALSE };
-static GtkWidget *mkisofs_path_entry, *mkhybrid_path_entry, *diff_path_entry;
+GtkWidget *prefs_speed_spin;
+misc_preferences misc_prefs = { TRUE, TRUE, TRUE, FALSE, FALSE, TRUE, FALSE, TRUE, FALSE, FALSE, TRUE, NULL};
+static GtkWidget *mkisofs_path_entry, *diff_path_entry;
 static GtkWidget *cdrecord_path_entry, *cdlabelgen_path_entry, *cdda2wav_path_entry;
+static GtkWidget *lpr_path_entry, *cdparanoia_path_entry;;
 static GtkWidget *prefs_enable_tooltips, *prefs_own_progresswin, *prefs_dummy;
-static GtkWidget *prefs_eject, *iso_location_entry, *rip_location_entry;
+static GtkWidget *prefs_eject, *iso_location_entry, *rip_location_entry, *fifo_entry;
+static GtkWidget *prefs_set_multisession, *prefs_old_session, *prefs_do_ms_check;
+static GtkWidget *selection_location_entry, *cddadev_entry, *isodev_entry;
+static GtkWidget *prefs_get_size, *prefs_beep, *prefs_burnproof, *prefs_disable_fixate;
+static GtkWidget *prefs_completion_msg;
 static config_cdr_data *prefs_cdr_info = NULL;
 
 static void new_mkisofs_path(GtkWidget *widget, gpointer data) {
@@ -49,11 +57,6 @@ static void new_mkisofs_path(GtkWidget *widget, gpointer data) {
 			gtk_file_selection_get_filename(data));
 	gtk_widget_destroy(GTK_WIDGET(data));
 }
-static void new_mkhybrid_path(GtkWidget *widget, gpointer data) {
-	gtk_entry_set_text(GTK_ENTRY(mkhybrid_path_entry),
-			gtk_file_selection_get_filename(data));
-	gtk_widget_destroy(GTK_WIDGET(data));
-}
 static void new_cdrecord_path(GtkWidget *widget, gpointer data) {
 	gtk_entry_set_text(GTK_ENTRY(cdrecord_path_entry),
 			gtk_file_selection_get_filename(data));
@@ -74,16 +77,26 @@ static void new_cdda2wav_path(GtkWidget *widget, gpointer data) {
 			gtk_file_selection_get_filename(data));
 	gtk_widget_destroy(GTK_WIDGET(data));
 }
+static void new_cdparanoia_path(GtkWidget *widget, gpointer data) {
+	gtk_entry_set_text(GTK_ENTRY(cdparanoia_path_entry), 
+			gtk_file_selection_get_filename(data));
+	gtk_widget_destroy(GTK_WIDGET(data));
+}
+static void new_lpr_path(GtkWidget *widget, gpointer data) {
+	gtk_entry_set_text(GTK_ENTRY(lpr_path_entry), 
+			gtk_file_selection_get_filename(data));
+	gtk_widget_destroy(GTK_WIDGET(data));
+}
 
 static GtkWidget *add_paths_page(void) {
-	GtkWidget *frame3, *table1, *button9, *button10;
-	GtkWidget *button11, *button12;
+	GtkWidget *frame3, *table1, *button9;
+	GtkWidget *button11, *button12, *button13;
 	file_req_info *add_info;
 	
 	frame3 = gtk_frame_new(NULL);
 	gtk_widget_show(frame3);
 
-	table1 = gtk_table_new(6, 2, FALSE);
+	table1 = gtk_table_new(8, 2, FALSE);
 	gtk_widget_show(table1);
 	gtk_container_add(GTK_CONTAINER(frame3), table1);
 	/* mkisofs */
@@ -93,15 +106,15 @@ static GtkWidget *add_paths_page(void) {
 			(GtkAttachOptions) GTK_FILL,
 			(GtkAttachOptions) GTK_FILL, 0, 0);
 
-	gtk_tooltips_set_tip(tooltips, button9, _("Open up filerequester "
-				"and sets the path to the mkisofs program "
+	gtk_tooltips_set_tip(tooltips, button9, _("Open up file selector "
+				"and set the path to the mkisofs program "
 				"(eg /usr/bin/mkisofs)"), NULL);
 	/* FIXME: this doesn't seem to be freed anywhere */
 	add_info = malloc(sizeof(file_req_info));
 	add_info->title = g_strdup(_("Select path to mkisofs"));
 	add_info->func = (gpointer) &new_mkisofs_path;
-      gtk_signal_connect(GTK_OBJECT(button9), "clicked",
-                  GTK_SIGNAL_FUNC(choose_path), (gpointer) add_info);
+	gtk_signal_connect(GTK_OBJECT(button9), "clicked",
+			GTK_SIGNAL_FUNC(choose_path), (gpointer) add_info);
 
 	mkisofs_path_entry = gtk_entry_new();
 	gtk_widget_show(mkisofs_path_entry);
@@ -109,28 +122,7 @@ static GtkWidget *add_paths_page(void) {
 	gtk_table_attach(GTK_TABLE(table1), mkisofs_path_entry, 1, 2, 0, 1,
 			(GtkAttachOptions) GTK_EXPAND | GTK_FILL,
 			(GtkAttachOptions) GTK_FILL, 0, 0);
-	/* mkhybrid */
-	button10 = gtk_button_new_with_label(_("mkhybrid"));
-	gtk_widget_show(button10);
-	gtk_table_attach(GTK_TABLE(table1), button10, 0, 1, 1, 2,
-			(GtkAttachOptions) GTK_FILL,
-			(GtkAttachOptions) GTK_FILL, 0, 0);
 
-	gtk_tooltips_set_tip(tooltips, button10, _("Open up filerequester "
-				"and sets the path to the mkhbrid program "
-				"(eg /usr/bin/mkhybrid)"), NULL);
-	add_info = malloc(sizeof(file_req_info));
-	add_info->title = g_strdup(_("Select path to mkhbyrd"));
-	add_info->func = (gpointer) &new_mkhybrid_path;
-      gtk_signal_connect(GTK_OBJECT(button10), "clicked",
-                  GTK_SIGNAL_FUNC(choose_path), (gpointer) add_info);
-
-	mkhybrid_path_entry = gtk_entry_new();
-	gtk_widget_show(mkhybrid_path_entry);
-	gtk_entry_set_text(GTK_ENTRY(mkhybrid_path_entry), mkhybrid_path);
-	gtk_table_attach(GTK_TABLE(table1), mkhybrid_path_entry, 1, 2, 1, 2,
-			GTK_EXPAND | GTK_FILL,
-			GTK_FILL, 0, 0);
 	/* cdrecord */
 	button11 = gtk_button_new_with_label(_("cdrecord"));
 	gtk_widget_show(button11);
@@ -138,14 +130,14 @@ static GtkWidget *add_paths_page(void) {
 			(GtkAttachOptions) GTK_FILL,
 			(GtkAttachOptions) GTK_FILL, 0, 0);
 
-	gtk_tooltips_set_tip(tooltips, button11, _("Open up filerequester "
-				"and sets the path to the cdrecord program "
+	gtk_tooltips_set_tip(tooltips, button11, _("Open up file selector "
+				"and set the path to the cdrecord program "
 				"(eg /usr/bin/cdrecord)"), NULL);
 	add_info = malloc(sizeof(file_req_info));
 	add_info->title = g_strdup(_("Select path to cdrecord"));
 	add_info->func = (gpointer) &new_cdrecord_path;
-      gtk_signal_connect(GTK_OBJECT(button11), "clicked",
-                  GTK_SIGNAL_FUNC(choose_path), (gpointer) add_info);
+	gtk_signal_connect(GTK_OBJECT(button11), "clicked",
+			GTK_SIGNAL_FUNC(choose_path), (gpointer) add_info);
 
 	cdrecord_path_entry = gtk_entry_new();
 	gtk_widget_show(cdrecord_path_entry);
@@ -160,14 +152,14 @@ static GtkWidget *add_paths_page(void) {
 			(GtkAttachOptions) GTK_FILL,
 			(GtkAttachOptions) GTK_FILL, 0, 0);
 
-	gtk_tooltips_set_tip(tooltips, button12, _("Open up filerequester "
-				"and sets the path to the cdlabelgen program "
+	gtk_tooltips_set_tip(tooltips, button12, _("Open up file selector "
+				"and set the path to the cdlabelgen program "
 				"(eg /usr/bin/cdlabelgen)"), NULL);
 	add_info = malloc(sizeof(file_req_info));
 	add_info->title = g_strdup(_("Select path to cdlabelgen"));
 	add_info->func = (gpointer) &new_cdlabelgen_path;
-      gtk_signal_connect(GTK_OBJECT(button12), "clicked",
-                  GTK_SIGNAL_FUNC(choose_path), (gpointer) add_info);
+	gtk_signal_connect(GTK_OBJECT(button12), "clicked",
+			GTK_SIGNAL_FUNC(choose_path), (gpointer) add_info);
 
 	cdlabelgen_path_entry = gtk_entry_new();
 	gtk_widget_show(cdlabelgen_path_entry);
@@ -182,14 +174,14 @@ static GtkWidget *add_paths_page(void) {
 			(GtkAttachOptions) GTK_FILL,
 			(GtkAttachOptions) GTK_FILL, 0, 0);
 
-	gtk_tooltips_set_tip(tooltips, button12, _("Open up filerequester "
-				"and sets the path to the diff program "
+	gtk_tooltips_set_tip(tooltips, button12, _("Open up file selector "
+				"and set the path to the diff program "
 				"(tested with GNU diff 2.7, others will probably work)"), NULL);
 	add_info = malloc(sizeof(file_req_info));
 	add_info->title = g_strdup(_("Select path to diff"));
 	add_info->func = (gpointer) &new_diff_path;
-      gtk_signal_connect(GTK_OBJECT(button12), "clicked",
-                  GTK_SIGNAL_FUNC(choose_path), (gpointer) add_info);
+	gtk_signal_connect(GTK_OBJECT(button12), "clicked",
+			GTK_SIGNAL_FUNC(choose_path), (gpointer) add_info);
 
 	diff_path_entry = gtk_entry_new();
 	gtk_widget_show(diff_path_entry);
@@ -204,14 +196,14 @@ static GtkWidget *add_paths_page(void) {
 			(GtkAttachOptions) GTK_FILL,
 			(GtkAttachOptions) GTK_FILL, 0, 0);
 
-	gtk_tooltips_set_tip(tooltips, button11, _("Open up filerequester "
-				"and sets the path to the cdda2wav program "
+	gtk_tooltips_set_tip(tooltips, button11, _("Open up file selector "
+				"and set the path to the cdda2wav program "
 				"(eg /usr/bin/cdda2wav)"), NULL);
 	add_info = malloc(sizeof(file_req_info));
 	add_info->title = g_strdup(_("Select path to cdda2wav"));
 	add_info->func = (gpointer) &new_cdda2wav_path;
-      gtk_signal_connect(GTK_OBJECT(button11), "clicked",
-                  GTK_SIGNAL_FUNC(choose_path), (gpointer) add_info);
+	gtk_signal_connect(GTK_OBJECT(button11), "clicked",
+			GTK_SIGNAL_FUNC(choose_path), (gpointer) add_info);
 
 	cdda2wav_path_entry = gtk_entry_new();
 	gtk_widget_show(cdda2wav_path_entry);
@@ -220,10 +212,59 @@ static GtkWidget *add_paths_page(void) {
 			GTK_EXPAND | GTK_FILL,
 			GTK_FILL, 0, 0);
 
+	/* cdparanoia */
+
+	button13 = gtk_button_new_with_label(_("cdparanoia"));
+	gtk_widget_show(button13);
+	gtk_table_attach(GTK_TABLE(table1), button13, 0, 1, 6, 7,
+			(GtkAttachOptions) GTK_FILL,
+			(GtkAttachOptions) GTK_FILL, 0, 0);
+
+	gtk_tooltips_set_tip(tooltips, button13, _("Open up file selector "
+				"and set the path to the cdparanoia program "
+				"(eg /usr/bin/cdparanoia)"), NULL);
+	add_info = malloc(sizeof(file_req_info));
+	add_info->title = g_strdup(_("Select path to cdparanoia"));
+	add_info->func = (gpointer) &new_cdparanoia_path;
+	gtk_signal_connect(GTK_OBJECT(button13), "clicked",
+			GTK_SIGNAL_FUNC(choose_path), (gpointer) add_info);
+
+	cdparanoia_path_entry = gtk_entry_new();
+	gtk_widget_show(cdparanoia_path_entry);
+	gtk_entry_set_text(GTK_ENTRY(cdparanoia_path_entry), cdparanoia_path);
+	gtk_table_attach(GTK_TABLE(table1), cdparanoia_path_entry, 1, 2, 6, 7,
+			GTK_EXPAND | GTK_FILL,
+			GTK_FILL, 0, 0);
+
+	/* lpr */
+	button11 = gtk_button_new_with_label(_("lpr"));
+	gtk_widget_show(button11);
+	gtk_table_attach(GTK_TABLE(table1), button11, 0, 1, 7, 8,
+			(GtkAttachOptions) GTK_FILL,
+			(GtkAttachOptions) GTK_FILL, 0, 0);
+
+	gtk_tooltips_set_tip(tooltips, button11, _("Open up filerequester "
+				"and set the path to the lpr program "
+				"(eg /usr/bin/lpr). Another method for printing "
+				"can also be use (e.g. pdq)."), NULL);
+	add_info = malloc(sizeof(file_req_info));
+	add_info->title = g_strdup(_("Select path to lpr"));
+	add_info->func = (gpointer) &new_lpr_path;
+	gtk_signal_connect(GTK_OBJECT(button11), "clicked",
+			GTK_SIGNAL_FUNC(choose_path), (gpointer) add_info);
+
+	lpr_path_entry = gtk_entry_new();
+	gtk_widget_show(lpr_path_entry);
+	gtk_entry_set_text(GTK_ENTRY(lpr_path_entry), lpr_path);
+	gtk_table_attach(GTK_TABLE(table1), lpr_path_entry, 1, 2, 7, 8,
+			GTK_EXPAND | GTK_FILL,
+			GTK_FILL, 0, 0);
+
 	return frame3;
 }
 static GtkWidget *add_devices_page(void) {
-	GtkWidget *swab_audio, *hbox, *vbox, *sep;
+	GtkWidget *swab_audio, *hbox, *vbox, *sep, *fifo_label;
+	GtkWidget *cddadev_label, *isodev_label;
 	int i;
 
 	prefs_cdr_info = config_cdr(0);
@@ -267,23 +308,73 @@ static GtkWidget *add_devices_page(void) {
 			"(little-endian) order"), NULL);
 	prefs_cdr_info->swab_audio = swab_audio;
 
-	select_speed(hbox, prefs_speed);
+	prefs_speed_spin = select_speed(hbox);
 
-	for (i = 0; i < NO_SPEED; i++) {
-		if (GTK_TOGGLE_BUTTON(speed_x[i])->active)
-			gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(prefs_speed[i]), TRUE);
-	}
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_speed_spin),
+			gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(speed_spin)));
+
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_widget_show(hbox);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
+
+	fifo_label = gtk_label_new(_("FIFO size: "));
+	gtk_widget_show(fifo_label);
+	gtk_box_pack_start(GTK_BOX(hbox), fifo_label, FALSE, FALSE, 0);
+	fifo_entry = gtk_entry_new();
+	gtk_widget_show(fifo_entry);
+	gtk_box_pack_start(GTK_BOX(hbox), fifo_entry, FALSE, FALSE, 0);
+	gtk_tooltips_set_tip(tooltips, fifo_entry, _("Set the fifo size (default "
+				"is 4 MB, and it will be used if nothing is specified) - goes "
+				"directly to the cdrecord fs= option so the syntax is the same "
+				"as cdrecords (e.g. 10x53k == 630 kBytes)"), NULL);
+
+	gtk_entry_set_text(GTK_ENTRY(fifo_entry),
+			gtk_entry_get_text(GTK_ENTRY(mainptr->cdr_option_info->fifo)));
+	prefs_cdr_info->fifo = fifo_entry;
+
+	/* cdrom device path settings */
+	sep = gtk_hseparator_new();
+	gtk_widget_show(sep);
+	gtk_box_pack_start(GTK_BOX(vbox), sep, TRUE, TRUE, 0);
+
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_widget_show(hbox);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
+
+	cddadev_label = gtk_label_new(_("CDDA device: "));
+	gtk_widget_show(cddadev_label);
+	gtk_box_pack_start(GTK_BOX(hbox), cddadev_label, FALSE, FALSE, 0);
+
+	cddadev_entry = gtk_entry_new();
+	gtk_widget_show(cddadev_entry);
+	gtk_box_pack_start(GTK_BOX(hbox), cddadev_entry, FALSE, FALSE, 0);
+	gtk_tooltips_set_tip(tooltips, cddadev_entry, _("The path to audio cdrom to read from "
+				"(for example /dev/cdrom or 0,6,0)."), NULL);
+
+	gtk_entry_set_text(GTK_ENTRY(cddadev_entry), cdda_device_path);
+
+	isodev_label = gtk_label_new(_("cdrom device: "));
+	gtk_widget_show(isodev_label);
+	gtk_box_pack_start(GTK_BOX(hbox), isodev_label, FALSE, FALSE, 0);
+
+	isodev_entry = gtk_entry_new();
+	gtk_widget_show(isodev_entry);
+	gtk_box_pack_start(GTK_BOX(hbox), isodev_entry, FALSE, FALSE, 0);
+	gtk_tooltips_set_tip(tooltips, isodev_entry, _("The path to existing ISO cdrom "
+				"(for example /dev/cdrom or 0,6,0)."), NULL);
+
+	gtk_entry_set_text(GTK_ENTRY(isodev_entry), isocdrom_device_path);
 
 	return prefs_cdr_info->widget;
 }
 
 static GtkWidget *add_misc_page(void) {
-	GtkWidget *frame, *table;
+	GtkWidget *frame, *table, *label;
 
 	frame = gtk_frame_new(NULL);
 	gtk_widget_show(frame);
 
-	table = gtk_table_new(2, 2, FALSE);
+	table = gtk_table_new(7, 2, FALSE);
 	gtk_widget_show(table);
 	gtk_container_add(GTK_CONTAINER(frame), table);
 
@@ -309,19 +400,95 @@ static GtkWidget *add_misc_page(void) {
 	gtk_table_attach(GTK_TABLE(table), prefs_dummy, 0, 1, 1, 2,
 			GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
 
-	prefs_eject = gtk_check_button_new_with_label(_("Eject enabled by default"));
+	prefs_eject = gtk_check_button_new_with_label(_("Eject target enabled by default"));
 	gtk_widget_show(prefs_eject);
 	gtk_tooltips_set_tip(tooltips, prefs_eject,
-			_("Have the eject checkbox ticked when starting gcombust."), NULL);	
+			_("Have the eject target checkbox ticked when starting gcombust."), NULL);	
 	gtk_table_attach(GTK_TABLE(table), prefs_eject, 1, 2, 1, 2,
 			GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
 
+	prefs_set_multisession =
+		gtk_check_button_new_with_label(_("Multisession enabled by default"));
+	gtk_widget_show(prefs_set_multisession);
+	gtk_tooltips_set_tip(tooltips, prefs_set_multisession,
+			_("Have the multisession run checkbox ticked when starting gcombust."), NULL);
+	gtk_table_attach(GTK_TABLE(table), prefs_set_multisession, 0, 1, 2, 3,
+			GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
+
+	prefs_do_ms_check = gtk_check_button_new_with_label(_("Validate multisession disks"));
+	gtk_widget_show(prefs_do_ms_check);
+	gtk_tooltips_set_tip(tooltips, prefs_do_ms_check,
+			_("When performing multisession operations, try to check that the CD "
+				"containing the old session is correct (not available on all "
+				"platforms)."),
+			NULL);	
+	gtk_table_attach(GTK_TABLE(table), prefs_do_ms_check, 1, 2, 2, 3,
+			GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
+
+	prefs_get_size = gtk_check_button_new_with_label(_("Get size enabled by default"));
+	gtk_widget_show(prefs_get_size);
+	gtk_tooltips_set_tip(tooltips, prefs_get_size,
+			_("Have the \"get size\" checkbox ticked when starting gcombust."), NULL);	
+	gtk_table_attach(GTK_TABLE(table), prefs_get_size, 0, 1, 3, 4,
+			GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
+
+	prefs_beep = gtk_check_button_new_with_label(_("Beep"));
+	gtk_widget_show(prefs_beep);
+	gtk_tooltips_set_tip(tooltips, prefs_beep,
+			_("Emit a beep when operations are completed."), NULL);	
+	gtk_table_attach(GTK_TABLE(table), prefs_beep, 1, 2, 3, 4,
+			GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
+
+	prefs_burnproof = gtk_check_button_new_with_label(_("Burnproof"));
+	gtk_widget_show(prefs_burnproof);
+	gtk_tooltips_set_tip(tooltips, prefs_burnproof,
+			_("Have the \"Burnproof\" checkbox ticked when starting gcombust."), NULL);	
+	gtk_table_attach(GTK_TABLE(table), prefs_burnproof, 0, 1, 4, 5,
+			GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
+
+	prefs_disable_fixate = gtk_check_button_new_with_label(_("Disable fixate"));
+	gtk_widget_show(prefs_disable_fixate);
+	gtk_tooltips_set_tip(tooltips, prefs_disable_fixate,
+			_("Have the \"Disable fixate\" checkbox ticked when starting gcombust."), NULL);	
+	gtk_table_attach(GTK_TABLE(table), prefs_disable_fixate, 1, 2, 4, 5,
+			GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
+
+	label = gtk_label_new(_("Path to old session: "));
+	gtk_widget_show(label);
+	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 5, 6,
+			GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
+
+	prefs_old_session = gtk_entry_new();
+	gtk_widget_show(prefs_old_session);
+	gtk_tooltips_set_tip(tooltips, prefs_old_session,
+			_("Default path to the old session when starting gcombust."), NULL);    
+	gtk_table_attach(GTK_TABLE(table), prefs_old_session, 1, 2, 5, 6,
+			GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
+
+	prefs_completion_msg = gtk_check_button_new_with_label(_("Completion message"));
+	gtk_widget_show(prefs_completion_msg);
+	gtk_tooltips_set_tip(tooltips, prefs_completion_msg,
+			_("Show a pop-up dialog upon completion of operation"), NULL);	
+	gtk_table_attach(GTK_TABLE(table), prefs_completion_msg, 0, 1, 6, 7,
+			GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
+
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_enable_tooltips),
 			misc_prefs.enable_tooltips);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_own_progresswin),
 			misc_prefs.own_progresswin);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_dummy), misc_prefs.dummy);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_eject), misc_prefs.eject);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_set_multisession),
+			misc_prefs.set_multisession);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_do_ms_check),
+			misc_prefs.do_ms_check);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_get_size), misc_prefs.get_size);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_beep), misc_prefs.beep);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_burnproof), misc_prefs.burnproof);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_disable_fixate), misc_prefs.disable_fixate);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_completion_msg), misc_prefs.completion_msg);
+	if (NULL != misc_prefs.old_session)
+		gtk_entry_set_text(GTK_ENTRY(prefs_old_session), misc_prefs.old_session);
 
 	return frame;
 }
@@ -341,8 +508,8 @@ static void add_path_location(char *name, char *title, char *tooltip, int n, Gtk
 	add_info = malloc(sizeof(file_req_info));
 	add_info->title = g_strdup(title);
 	add_info->func = sel_function;
-      gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                  GTK_SIGNAL_FUNC(choose_path), (gpointer) add_info);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			GTK_SIGNAL_FUNC(choose_path), (gpointer) add_info);
 
 	gtk_widget_show(entry);
 	gtk_entry_set_text(GTK_ENTRY(entry), default_path);
@@ -361,6 +528,11 @@ static void new_rip_location(GtkWidget *widget, gpointer data) {
 			gtk_file_selection_get_filename(data));
 	gtk_widget_destroy(GTK_WIDGET(data));
 }
+static void new_selection_location(GtkWidget *widget, gpointer data) {
+	gtk_entry_set_text(GTK_ENTRY(selection_location_entry), 
+			gtk_file_selection_get_filename(data));
+	gtk_widget_destroy(GTK_WIDGET(data));
+}
 
 static GtkWidget *add_storage_page(void) {
 	GtkWidget *frame, *table;
@@ -373,7 +545,7 @@ static GtkWidget *add_storage_page(void) {
 	gtk_container_add(GTK_CONTAINER(frame), table);
 
 	iso_location_entry = gtk_entry_new();
-	add_path_location(_("ISO"), _("Select default file to write ISO images to"),
+	add_path_location(_("ISO"), _("Select default file (or dir if ending with /) to write ISO images to"),
 			_("Default file to write ISO images to."), 0, table, iso_location_entry, "",
 			(gpointer) &new_iso_location);
 	gtk_entry_set_text(GTK_ENTRY(iso_location_entry), def_image_path);
@@ -386,6 +558,12 @@ static GtkWidget *add_storage_page(void) {
 		def_rip_path = g_strdup("");
 	gtk_entry_set_text(GTK_ENTRY(rip_location_entry), def_rip_path);
 
+	selection_location_entry = gtk_entry_new();
+	add_path_location(_("Selection"), _("Select default directory where file selection profiles are stored."),
+			_("Default directory for file selection profiles.."), 2, table, selection_location_entry, "",
+			(gpointer) &new_selection_location);
+	gtk_entry_set_text(GTK_ENTRY(selection_location_entry), def_selection_path);
+
 	return frame;
 }
 
@@ -400,6 +578,7 @@ static void check_path(char *path) {
 		g_snprintf(message, strlen(m) + strlen(path), m, path);
 		alert_user_of_error(message);
 		free(message);
+		return;
       }
 	if (S_ISDIR(buf.st_mode)) {
 		char *m = _(" %s is a directory, \n it should be "
@@ -415,11 +594,12 @@ static void check_path(char *path) {
 static void check_all_paths_set(void) {
 	/* TODO: check versions */
 	check_path(mkisofs_path);
-	check_path(mkhybrid_path);
 	check_path(cdrecord_path);
 	check_path(cdlabelgen_path);
 	check_path(diff_path);
 	check_path(cdda2wav_path);
+	check_path(cdparanoia_path);
+	check_path(lpr_path);
 }
 
 static int destroy_prefs_win(GtkWidget *widget, gpointer data) {
@@ -443,11 +623,6 @@ static void accept_path_changes(void) {
 		mkisofs_path = g_strdup(path);
 	}
 
-	if (NULL != (path = gtk_entry_get_text(GTK_ENTRY(mkhybrid_path_entry)))) {
-		free(mkhybrid_path);
-		mkhybrid_path = g_strdup(path);
-	}
-
 	if (NULL != (path = gtk_entry_get_text(GTK_ENTRY(cdrecord_path_entry)))) {
 		free(cdrecord_path);
 		cdrecord_path = g_strdup(path);
@@ -468,11 +643,20 @@ static void accept_path_changes(void) {
 		cdda2wav_path = g_strdup(path);
 	}
 
+	if (NULL != (path = gtk_entry_get_text(GTK_ENTRY(cdparanoia_path_entry)))) {
+		free(cdparanoia_path);
+		cdparanoia_path = g_strdup(path);
+	}
+
+	if (NULL != (path = gtk_entry_get_text(GTK_ENTRY(lpr_path_entry)))) {
+		free(lpr_path);
+		lpr_path = g_strdup(path);
+	}
+
 	check_all_paths_set();
 }
 
 void apply_misc_changes(void) {
-
 	if (misc_prefs.enable_tooltips)
 		gtk_tooltips_enable(tooltips);
 	else
@@ -485,9 +669,13 @@ void apply_misc_changes(void) {
 
 	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(dummy), misc_prefs.dummy);
 	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(eject), misc_prefs.eject);
-
-	if (NULL != cdda_device_path)
-		gtk_entry_set_text(GTK_ENTRY(audio_device_path), cdda_device_path);
+	if (NULL != misc_prefs.old_session)
+		gtk_entry_set_text (GTK_ENTRY(old_session), misc_prefs.old_session);
+	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(enable_multisession),
+			misc_prefs.set_multisession);
+	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(need_track_size), misc_prefs.get_size);
+	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(enable_burnproof), misc_prefs.burnproof);
+	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(disable_fixate), misc_prefs.disable_fixate);
 }
 
 static void accept_misc_changes(void) {
@@ -495,6 +683,17 @@ static void accept_misc_changes(void) {
 	misc_prefs.own_progresswin = GTK_TOGGLE_BUTTON(prefs_own_progresswin)->active;
 	misc_prefs.dummy = GTK_TOGGLE_BUTTON(prefs_dummy)->active;
 	misc_prefs.eject = GTK_TOGGLE_BUTTON(prefs_eject)->active;
+	misc_prefs.set_multisession = GTK_TOGGLE_BUTTON(prefs_set_multisession)->active;
+	misc_prefs.do_ms_check = GTK_TOGGLE_BUTTON(prefs_do_ms_check)->active;
+	misc_prefs.get_size = GTK_TOGGLE_BUTTON(prefs_get_size)->active;
+	misc_prefs.beep = GTK_TOGGLE_BUTTON(prefs_beep)->active;
+	misc_prefs.burnproof = GTK_TOGGLE_BUTTON(prefs_burnproof)->active;
+	misc_prefs.disable_fixate = GTK_TOGGLE_BUTTON(prefs_disable_fixate)->active;
+	misc_prefs.completion_msg = GTK_TOGGLE_BUTTON(prefs_completion_msg)->active;
+
+	if (NULL != misc_prefs.old_session)
+		g_free(misc_prefs.old_session);
+	misc_prefs.old_session = g_strdup(gtk_entry_get_text(GTK_ENTRY(prefs_old_session)));
 
 	apply_misc_changes();
 }
@@ -522,10 +721,19 @@ static void accept_scsi_changes(void) {
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mainptr->cdr_option_info->swab_audio),
 			GTK_TOGGLE_BUTTON(prefs_cdr_info->swab_audio)->active ? TRUE : FALSE);
 
-	for (i = 0; i < NO_SPEED; i++) {
-		if (GTK_TOGGLE_BUTTON(prefs_speed[i])->active)
-			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(speed_x[i]), TRUE);
-	}
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(speed_spin),
+			gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(prefs_speed_spin)));
+
+	gtk_entry_set_text(GTK_ENTRY(mainptr->cdr_option_info->fifo),
+			gtk_entry_get_text(GTK_ENTRY(prefs_cdr_info->fifo)));
+
+	g_free(cdda_device_path);
+	cdda_device_path = g_strdup(gtk_entry_get_text(GTK_ENTRY(cddadev_entry)));
+	gtk_entry_set_text(GTK_ENTRY(audio_device_path), cdda_device_path);
+
+	g_free(isocdrom_device_path);
+	isocdrom_device_path = g_strdup(gtk_entry_get_text(GTK_ENTRY(isodev_entry)));
+	gtk_entry_set_text(GTK_ENTRY(iso_device_path), isocdrom_device_path);
 }
 
 void apply_storage_changes(void) {
@@ -545,6 +753,11 @@ static void accept_storage_changes(void) {
 		def_rip_path = g_strdup(path);
 	}
 
+	if (NULL != (path = gtk_entry_get_text(GTK_ENTRY(selection_location_entry)))) {
+		g_free(def_selection_path);
+		def_selection_path = g_strdup(path);
+	}
+
 	apply_storage_changes();
 }
 
@@ -614,18 +827,18 @@ void preferences_cmd_callback(GtkWidget *widget, gpointer data) {
 	gtk_widget_show(cancel);
 	gtk_tooltips_set_tip(tooltips, cancel, _("Forget changes"), NULL);
 	gtk_box_pack_end(GTK_BOX(hbox), cancel, FALSE, FALSE, 0);
-      gtk_signal_connect(GTK_OBJECT(cancel), "clicked",
-                  GTK_SIGNAL_FUNC(forget_changes), (gpointer) prefs_win);
+	gtk_signal_connect(GTK_OBJECT(cancel), "clicked",
+			GTK_SIGNAL_FUNC(forget_changes), (gpointer) prefs_win);
 
 	ok = gtk_button_new_with_label(_("Ok"));
 	gtk_widget_show(ok);
 	gtk_tooltips_set_tip(tooltips, ok, _("Accept and save changes"), NULL);
 	gtk_box_pack_end(GTK_BOX(hbox), ok, FALSE, FALSE, 0);
-      gtk_signal_connect(GTK_OBJECT(ok), "clicked",
-                  GTK_SIGNAL_FUNC(accept_changes), (gpointer) prefs_win);
+	gtk_signal_connect(GTK_OBJECT(ok), "clicked",
+			GTK_SIGNAL_FUNC(accept_changes), (gpointer) prefs_win);
 
-      gtk_signal_connect(GTK_OBJECT(prefs_win), "delete_event",
-                  GTK_SIGNAL_FUNC(destroy_prefs_win), NULL);
+	gtk_signal_connect(GTK_OBJECT(prefs_win), "delete_event",
+			GTK_SIGNAL_FUNC(destroy_prefs_win), NULL);
 
 	gtk_widget_show(prefs_win);
 }
