@@ -16,15 +16,27 @@
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
+typedef enum {
+	false,
+	true
+} bool;
+
 typedef struct {
-	int enable_tooltips;
-	int own_progresswin;
-	int dummy;
-	int eject;
+	bool enable_tooltips;
+	bool own_progresswin;
+	bool dummy;
+	bool eject;
+	bool set_multisession;
+	bool do_ms_check;
+	bool get_size;
+	bool beep;
+	bool burnproof;
+	bool disable_fixate;
+	bool completion_msg;
+	char *old_session;
 } misc_preferences;
 
 extern misc_preferences misc_prefs;
-extern GtkWidget *prefs_speed[];
 
 void apply_misc_changes(void);
 void apply_storage_changes(void);
